@@ -1,12 +1,15 @@
 from __future__ import annotations
 
 import warnings
+from typing import TYPE_CHECKING
 
 import pytest
-from pytest_benchmark.fixture import BenchmarkFixture
 
 import ry
 
+if TYPE_CHECKING:
+    from pytest_benchmark.fixture import BenchmarkFixture
+
 
 def _warn_benchmarking_with_debug_build() -> None:
     warnings.warn("ry is built in debug mode", UserWarning, stacklevel=2)
