@@ -3,9 +3,9 @@ use jiff::tz;
 use pyo3::prelude::*;
 
 const JIFF_ERA_STRINGS: &str = "'compatible', 'earlier', 'later', 'reject'";
-impl FromPyObject<'_> for JiffTzDisambiguation {
-    fn extract_bound(ob: &Bound<'_, PyAny>) -> PyResult<Self> {
-        // downcast to string...
+impl<'py> FromPyObject<'_, 'py> for JiffTzDisambiguation {
+    type Error = PyErr;
+    fn extract(ob: Borrowed<'_, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(s) = ob.extract::<&str>() {
             match s {
                 "compatible" => Ok(tz::Disambiguation::Compatible.into()),
