@@ -1,8 +1,8 @@
-use crate::{RyDate, RyDateTime, RyTime, RyTimestamp};
+use crate::{RyDate, RyDateTime, RyTime, RyTimestamp, RyZoned};
 use parking_lot::Mutex;
 use pyo3::prelude::*;
 
-#[pyclass(name = "DateSeries", frozen)]
+#[pyclass(name = "DateSeries", frozen, immutable_type)]
 #[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct RyDateSeries {
     pub(crate) series: Mutex<jiff::civil::DateSeries>,
@@ -27,13 +27,15 @@ impl RyDateSeries {
     }
 
     #[pyo3(signature = (n = 1))]
-    fn take(&self, n: usize) -> Vec<RyDate> {
-        let mut s = self.series.lock();
-        s.by_ref().take(n).map(RyDate::from).collect()
+    fn take(&self, py: Python<'_>, n: usize) -> Vec<RyDate> {
+        py.detach(|| {
+            let mut s = self.series.lock();
+            s.by_ref().take(n).map(RyDate::from).collect()
+        })
     }
 }
 
-#[pyclass(name = "DateTimeSeries", frozen)]
+#[pyclass(name = "DateTimeSeries", frozen, immutable_type)]
 #[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct RyDateTimeSeries {
     pub(crate) series: Mutex<jiff::civil::DateTimeSeries>,
@@ -58,13 +60,15 @@ impl RyDateTimeSeries {
     }
 
     #[pyo3(signature = (n = 1))]
-    fn take(&self, n: usize) -> Vec<RyDateTime> {
-        let mut s = self.series.lock();
-        s.by_ref().take(n).map(RyDateTime::from).collect()
+    fn take(&self, py: Python<'_>, n: usize) -> Vec<RyDateTime> {
+        py.detach(|| {
+            let mut s = self.series.lock();
+            s.by_ref().take(n).map(RyDateTime::from).collect()
+        })
     }
 }
 
-#[pyclass(name = "TimeSeries", frozen)]
+#[pyclass(name = "TimeSeries", frozen, immutable_type)]
 #[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct RyTimeSeries {
     pub(crate) series: Mutex<jiff::civil::TimeSeries>,
@@ -88,14 +92,15 @@ impl RyTimeSeries {
         self.series.lock().next().map(RyTime::from)
     }
 
-    #[pyo3(signature = (n = 1))]
-    fn take(&self, n: usize) -> Vec<RyTime> {
-        let mut s = self.series.lock();
-        s.by_ref().take(n).map(RyTime::from).collect()
+    fn take(&self, py: Python<'_>, n: usize) -> Vec<RyTime> {
+        py.detach(|| {
+            let mut s = self.series.lock();
+            s.by_ref().take(n).map(RyTime::from).collect()
+        })
     }
 }
 
-#[pyclass(name = "TimestampSeries", frozen)]
+#[pyclass(name = "TimestampSeries", frozen, immutable_type)]
 #[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct RyTimestampSeries {
     pub(crate) series: Mutex<jiff::TimestampSeries>,
@@ -120,8 +125,43 @@ impl RyTimestampSeries {
     }
 
     #[pyo3(signature = (n = 1))]
-    fn take(&self, n: usize) -> Vec<RyTimestamp> {
-        let mut s = self.series.lock();
-        s.by_ref().take(n).map(RyTimestamp::from).collect()
+    fn take(&self, py: Python<'_>, n: usize) -> Vec<RyTimestamp> {
+        py.detach(|| {
+            let mut s = self.series.lock();
+            s.by_ref().take(n).map(RyTimestamp::from).collect()
+        })
+    }
+}
+
+#[pyclass(name = "ZonedSeries", frozen, immutable_type)]
+#[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
+pub struct RyZonedSeries {
+    pub(crate) series: Mutex<jiff::ZonedSeries>,
+}
+
+impl From<jiff::ZonedSeries> for RyZonedSeries {
+    fn from(series: jiff::ZonedSeries) -> Self {
+        Self {
+            series: Mutex::new(series),
+        }
+    }
+}
+
+#[pymethods]
+impl RyZonedSeries {
+    fn __iter__(slf: PyRef<'_, Self>) -> PyRef<'_, Self> {
+        slf
+    }
+
+    fn __next__(&self) -> Option<RyZoned> {
+        self.series.lock().next().map(RyZoned::from)
+    }
+
+    #[pyo3(signature = (n = 1))]
+    fn take(&self, py: Python<'_>, n: usize) -> Vec<RyZoned> {
+        py.detach(|| {
+            let mut s = self.series.lock();
+            s.by_ref().take(n).map(RyZoned::from).collect()
+        })
     }
 }
