@@ -6,8 +6,9 @@ use ryo3_core::types::PathLike;
 
 /// Return `True` if pathlike points to same file
 #[pyfunction]
-pub fn is_same_file(p1: PathLike, p2: PathLike) -> PyResult<bool> {
-    Ok(same_file::is_same_file(p1, p2)?)
+pub fn is_same_file(py: Python<'_>, p1: PathLike, p2: PathLike) -> PyResult<bool> {
+    py.detach(|| same_file::is_same_file(p1, p2))
+        .map_err(|e| pyo3::exceptions::PyOSError::new_err(format!("is_same_file error: {e}")))
 }
 
 pub fn pymod_add(m: &Bound<'_, PyModule>) -> PyResult<()> {
