@@ -3,6 +3,14 @@ use crate::constants::{Depth, MAX_DEPTH};
 use crate::errors::pyerr2sererr;
 use crate::ob_type::PyObType;
 use crate::ser::PySerializeContext;
+#[cfg(any(
+    feature = "ryo3-http",
+    feature = "ryo3-jiff",
+    feature = "ryo3-ulid",
+    feature = "ryo3-url",
+    feature = "ryo3-uuid",
+    feature = "ryo3-std"
+))]
 use crate::ser::rytypes;
 use crate::ser::safe_impl::{
     SerializePyBool, SerializePyBytesLike, SerializePyDataclass, SerializePyDate,
