@@ -3,7 +3,7 @@ use pyo3::prelude::*;
 use pyo3::types::PyTuple;
 use pyo3::{IntoPyObjectExt, intern};
 
-#[pyclass(name = "SizeFormatter", frozen)]
+#[pyclass(name = "SizeFormatter", frozen, immutable_type)]
 #[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct PySizeFormatter {
     formatter: size::fmt::SizeFormatter,
