@@ -1,12 +1,16 @@
+#[cfg(feature = "jiff")]
+use jiff::fmt::friendly::Designator;
 use pyo3::basic::CompareOp;
-use pyo3::prelude::PyAnyMethods;
-use pyo3::types::{PyDict, PyInt, PyTuple};
-use pyo3::{Bound, FromPyObject, IntoPyObjectExt, PyAny, PyResult, Python, pyclass, pymethods};
+use pyo3::types::{PyAnyMethods, PyDict, PyDictMethods, PyInt, PyTuple};
+use pyo3::{Bound, IntoPyObjectExt, PyAny, PyResult, Python, pyclass, pymethods};
+use ryo3_core::{PyFromStr, PyParse};
 use ryo3_macro_rules::{
-    py_overflow_err, py_overflow_error, py_type_err, py_value_err, py_zero_division_err,
+    py_key_err, py_overflow_err, py_overflow_error, py_type_err, py_value_err, py_zero_division_err,
 };
 use std::hash::{DefaultHasher, Hash, Hasher};
 use std::ops::{Div, Mul};
+#[cfg(feature = "jiff")]
+use std::str::FromStr;
 use std::time::Duration;
 
 const NANOS_PER_SEC: u32 = 1_000_000_000;
@@ -15,10 +19,44 @@ const MINS_PER_HOUR: u64 = 60;
 const HOURS_PER_DAY: u64 = 24;
 const DAYS_PER_WEEK: u64 = 7;
 const MAX_DAYS: u64 = u64::MAX / (SECS_PER_MINUTE * MINS_PER_HOUR * HOURS_PER_DAY);
-const MAX_WEEKS: u64 = u64::MAX / (SECS_PER_MINUTE * MINS_PER_HOUR * HOURS_PER_DAY * DAYS_PER_WEEK);
+const SECS_PER_WEEK: u64 = SECS_PER_MINUTE * MINS_PER_HOUR * HOURS_PER_DAY * DAYS_PER_WEEK;
+
+// jiff
+#[cfg(feature = "jiff")]
+const TEMPORAL_SPAN_PARSER: jiff::fmt::temporal::SpanParser =
+    jiff::fmt::temporal::SpanParser::new();
+#[cfg(feature = "jiff")]
+const TEMPORAL_SPAN_PRINTER: jiff::fmt::temporal::SpanPrinter =
+    jiff::fmt::temporal::SpanPrinter::new();
+#[cfg(feature = "jiff")]
+const FRIENDLY_SPAN_PARSER: jiff::fmt::friendly::SpanParser =
+    jiff::fmt::friendly::SpanParser::new();
+
+// Maybe use?`HumanTime` designator for friendly parser/printer to avoid ambiguous `µ`:w
+// REF: https://github.com/jessekrubin/ry/discussions/229#discussioncomment-14928815
+// ```txt
+// RUF001 String contains ambiguous `µ` (MICRO SIGN). Did you mean `μ` (GREEK SMALL LETTER MU)?
+//   --> tests\std\test_duration_str.py:45:59
+//    |
+// 43 |         max_dur = ry.Duration.MAX
+// 44 |         iso_str = max_dur.friendly()
+// 45 |         assert iso_str == "5124095576030431h 15s 999ms 999µs 999ns"
+//    |                                                           ^
+// 46 |         parsed_max_dur = ry.Duration.from_str(iso_str)
+// 47 |         assert parsed_max_dur == max_dur
+//    |
+//
+// Found 1 error.
+// ```
+#[cfg(feature = "jiff")]
+const FRIENDLY_SPAN_PRINTER: jiff::fmt::friendly::SpanPrinter =
+    jiff::fmt::friendly::SpanPrinter::new();
+// const FRIENDLY_SPAN_PRINTER: jiff::fmt::friendly::SpanPrinter = jiff::fmt::friendly::SpanPrinter::new().designator(
+// jiff::fmt::friendly::Designator::HumanTime,
+// );
 
 #[derive(Copy, Clone, PartialEq)]
-#[pyclass(name = "Duration", frozen)]
+#[pyclass(name = "Duration", frozen, immutable_type)]
 #[cfg_attr(feature = "ry", pyo3(module = "ry.ryo3"))]
 pub struct PyDuration(pub Duration);
 
@@ -27,6 +65,7 @@ impl From<Duration> for PyDuration {
         Self(d)
     }
 }
+
 impl From<PyDuration> for Option<Duration> {
     fn from(d: PyDuration) -> Self {
         Some(d.0)
@@ -176,24 +215,35 @@ impl PyDuration {
     // ========================================================================
     // MATHS/OPERATORS
     // ========================================================================
-    fn __richcmp__(&self, other: PyDurationComparable, op: CompareOp) -> bool {
-        match other {
-            PyDurationComparable::PyDuration(other) => match op {
-                CompareOp::Eq => self.0 == other.0,
-                CompareOp::Ne => self.0 != other.0,
-                CompareOp::Lt => self.0 < other.0,
-                CompareOp::Le => self.0 <= other.0,
-                CompareOp::Gt => self.0 > other.0,
-                CompareOp::Ge => self.0 >= other.0,
-            },
-            PyDurationComparable::Duration(other) => match op {
-                CompareOp::Eq => self.0 == other,
-                CompareOp::Ne => self.0 != other,
-                CompareOp::Lt => self.0 < other,
-                CompareOp::Le => self.0 <= other,
-                CompareOp::Gt => self.0 > other,
-                CompareOp::Ge => self.0 >= other,
-            },
+    // OLD VERSION ALLOWING FOR DURATION/DELTA COMPARISONS
+    // fn __richcmp__(&self, other: PyDurationComparable, op: CompareOp) -> bool {
+    //     match other {
+    //         PyDurationComparable::PyDuration(other) => match op {
+    //             CompareOp::Eq => self.0 == other.0,
+    //             CompareOp::Ne => self.0 != other.0,
+    //             CompareOp::Lt => self.0 < other.0,
+    //             CompareOp::Le => self.0 <= other.0,
+    //             CompareOp::Gt => self.0 > other.0,
+    //             CompareOp::Ge => self.0 >= other.0,
+    //         },
+    //         PyDurationComparable::Duration(other) => match op {
+    //             CompareOp::Eq => self.0 == other,
+    //             CompareOp::Ne => self.0 != other,
+    //             CompareOp::Lt => self.0 < other,
+    //             CompareOp::Le => self.0 <= other,
+    //             CompareOp::Gt => self.0 > other,
+    //             CompareOp::Ge => self.0 >= other,
+    //         },
+    //     }
+    // }
+    fn __richcmp__(&self, other: &Self, op: CompareOp) -> bool {
+        match op {
+            CompareOp::Eq => self.0 == other.0,
+            CompareOp::Ne => self.0 != other.0,
+            CompareOp::Lt => self.0 < other.0,
+            CompareOp::Le => self.0 <= other.0,
+            CompareOp::Gt => self.0 > other.0,
+            CompareOp::Ge => self.0 >= other.0,
         }
     }
 
@@ -309,14 +359,30 @@ impl PyDuration {
         self.0.subsec_nanos()
     }
 
+    #[getter]
+    fn nanoseconds(&self) -> u32 {
+        self.0.subsec_nanos()
+    }
+
+    #[getter]
+    fn ns(&self) -> u32 {
+        self.0.subsec_nanos()
+    }
+
     #[getter]
     fn days(&self) -> u64 {
         self.0.as_secs() / 86400
     }
 
-    /// Return the number of seconds in the duration not counting days
+    /// Return the number of seconds in the duration counting days
     #[getter]
     fn seconds(&self) -> u64 {
+        self.0.as_secs()
+    }
+
+    /// Return the seconds % days (self.seconds % 86400)
+    #[getter]
+    fn seconds_remainder(&self) -> u64 {
         self.0.as_secs() % 86400
     }
 
@@ -402,9 +468,24 @@ impl PyDuration {
 
     #[staticmethod]
     fn from_dict(dict: &Bound<'_, PyDict>) -> PyResult<Self> {
-        let secs: u64 = dict.get_item(interns::secs(dict.py()))?.extract()?;
-        let nanos: u32 = dict.get_item(interns::nanos(dict.py()))?.extract()?;
-        Self::new(secs, nanos)
+        let secs = dict.get_item(interns::secs(dict.py()))?;
+        let nanos = dict.get_item(interns::nanos(dict.py()))?;
+        match (secs, nanos) {
+            (Some(secs), Some(nanos)) => {
+                let secs = secs.extract::<u64>()?;
+                let nanos = nanos.extract::<u32>()?;
+                Self::new(secs, nanos)
+            }
+            // (Some(secs), None) => {
+            //     let secs = secs.extract::<u64>()?;
+            //     Self::new(secs, 0)
+            // }
+            // (None, Some(nanos)) => {
+            //     let nanos = nanos.extract::<u32>()?;
+            //     Self::new(0, nanos)
+            // }
+            _ => py_key_err!("dict must contain 'secs' and/or 'nanos' keys"),
+        }
     }
 
     // ========================================================================
@@ -461,11 +542,11 @@ impl PyDuration {
 
     #[staticmethod]
     fn from_weeks(weeks: u64) -> PyResult<Self> {
-        if weeks > u64::MAX / (MAX_WEEKS) {
-            py_overflow_err!("overflow in Duration::from_weeks: {weeks} > {MAX_WEEKS}")
-        } else {
-            Ok(Self(Duration::from_secs(weeks * 60 * 60 * 24 * 7)))
-        }
+        weeks
+            .checked_mul(SECS_PER_WEEK)
+            .map(|v| Duration::from_secs(v).into())
+            .ok_or_else(|| py_overflow_error!("overflow in Duration::from_weeks"))
+        // Ok(Self(Duration::from_secs(total)))
     }
 
     #[staticmethod]
@@ -478,9 +559,86 @@ impl PyDuration {
         Self::try_from_secs_f64(secs)
     }
 
+    #[cfg(feature = "jiff")]
+    #[staticmethod]
+    fn fromisoformat(isoformat: &str) -> PyResult<Self> {
+        use jiff::fmt::temporal::SpanParser;
+        use ryo3_macro_rules::py_value_error;
+        let parser = SpanParser::new();
+        let duration = parser
+            .parse_unsigned_duration(isoformat)
+            .map_err(|e| py_value_error!("invalid isoformat string: {e}"))?;
+        Ok(Self(duration))
+    }
+
+    #[staticmethod]
+    fn from_str(s: &str) -> PyResult<Self> {
+        Self::py_from_str(s)
+    }
+
+    #[staticmethod]
+    fn parse(s: &Bound<'_, PyAny>) -> PyResult<Self> {
+        Self::py_parse(s)
+    }
+
     // ========================================================================
     // METHODS
     // ========================================================================
+    #[cfg(feature = "jiff")]
+    #[pyo3(name = "to_string")]
+    fn py_to_string(&self) -> String {
+        self.__str__()
+    }
+
+    #[cfg(feature = "jiff")]
+    fn __str__(&self) -> String {
+        self.isoformat()
+    }
+
+    #[cfg(feature = "jiff")]
+    fn __format__(&self, fmt: &str) -> PyResult<String> {
+        if fmt == "#" {
+            Ok(FRIENDLY_SPAN_PRINTER.unsigned_duration_to_string(&self.0))
+        } else if fmt.is_empty() {
+            Ok(self.py_to_string())
+        } else {
+            py_type_err!("Invalid format specifier '{fmt}' for Duration")
+        }
+    }
+
+    #[cfg(feature = "jiff")]
+    fn isoformat(&self) -> String {
+        TEMPORAL_SPAN_PRINTER.unsigned_duration_to_string(&self.0)
+    }
+
+    #[cfg(feature = "jiff")]
+    #[pyo3(signature = (designator = None))]
+    fn friendly(&self, designator: Option<&str>) -> PyResult<String> {
+        if let Some(designator) = designator {
+            match designator {
+                "human-time" | "human" => Ok(FRIENDLY_SPAN_PRINTER
+                    .designator(Designator::HumanTime)
+                    .unsigned_duration_to_string(&self.0)),
+                "short" => Ok(FRIENDLY_SPAN_PRINTER
+                    .designator(Designator::Short)
+                    .unsigned_duration_to_string(&self.0)),
+                "compact" => Ok(FRIENDLY_SPAN_PRINTER
+                    .designator(Designator::Compact)
+                    .unsigned_duration_to_string(&self.0)),
+                "verbose" => Ok(FRIENDLY_SPAN_PRINTER
+                    .designator(Designator::Verbose)
+                    .unsigned_duration_to_string(&self.0)),
+                other => {
+                    py_value_err!(
+                        "invalid designator: {other} (expected 'human'/'human-time', 'short', or 'compact')"
+                    )
+                }
+            }
+        } else {
+            Ok(FRIENDLY_SPAN_PRINTER.unsigned_duration_to_string(&self.0))
+        }
+    }
+
     #[pyo3(signature = (interval = None))]
     /// Sleep for the duration
     pub(crate) fn sleep(&self, py: Python<'_>, interval: Option<u64>) -> PyResult<()> {
@@ -500,12 +658,12 @@ impl PyDuration {
         let mut remaining = sleep_duration;
         while remaining > check_interval {
             py.check_signals()?; // This ensures signals are handled
-            std::thread::sleep(check_interval);
+            py.detach(|| std::thread::sleep(check_interval));
             remaining -= check_interval;
         }
         if remaining > Duration::ZERO {
             py.check_signals()?; // One last signal check before sleeping
-            std::thread::sleep(remaining);
+            py.detach(|| std::thread::sleep(remaining));
         }
 
         Ok(())
@@ -535,10 +693,18 @@ impl PyDuration {
     //     Ok(())
     // }
 
-    fn abs_diff(&self, other: PyDurationComparable) -> Self {
-        match other {
-            PyDurationComparable::PyDuration(other) => Self(self.0.abs_diff(other.0)),
-            PyDurationComparable::Duration(other) => Self(self.0.abs_diff(other)),
+    fn abs_diff(&self, other: Bound<'_, PyAny>) -> PyResult<Self> {
+        if let Ok(d) = other.cast_exact::<Self>() {
+            let rs_dur = d.get();
+            Ok(Self(self.0.abs_diff(rs_dur.0)))
+        } else if let Ok(d) = other.cast::<pyo3::types::PyDelta>() {
+            if let Ok(dur) = d.extract::<Duration>() {
+                Ok(Self(self.0.abs_diff(dur)))
+            } else {
+                py_value_err!("cannot compare with negative timedelta")
+            }
+        } else {
+            py_type_err!("unsupported operand type(s); must be Duration | datetime.timedelta")
         }
     }
 
@@ -645,6 +811,61 @@ impl PyDuration {
     fn saturating_sub(&self, other: &Self) -> Self {
         Self::from(self.0.saturating_sub(other.0))
     }
+
+    // ========================================================================
+    // PYDANTIC
+    // ========================================================================
+    #[cfg(feature = "pydantic")]
+    #[staticmethod]
+    fn from_any<'py>(value: &Bound<'py, PyAny>) -> PyResult<Bound<'py, PyAny>> {
+        let py = value.py();
+        if value.is_exact_instance_of::<Self>() {
+            value.into_bound_py_any(py)
+        } else if let Ok(pystr) = value.cast::<pyo3::types::PyString>() {
+            let s = pystr.extract::<&str>()?;
+            Self::from_str(s).map(|dt| dt.into_bound_py_any(py).map(Bound::into_any))?
+        } else if let Ok(pybytes) = value.cast::<pyo3::types::PyBytes>() {
+            use pyo3::types::PyBytesMethods;
+            let s = String::from_utf8_lossy(pybytes.as_bytes());
+            Self::from_str(&s).map(|dt| dt.into_bound_py_any(py).map(Bound::into_any))?
+        } else if let Ok(v) = value.cast_exact::<pyo3::types::PyFloat>() {
+            let f = v.extract::<f64>()?;
+            Self::try_from_secs_f64(f).map(|dt| dt.into_bound_py_any(py).map(Bound::into_any))?
+        } else if let Ok(v) = value.cast_exact::<pyo3::types::PyInt>() {
+            let i = v.extract::<u64>()?;
+            Self::from(Duration::new(i, 0)).into_bound_py_any(py)
+        } else if let Ok(d) = value.extract::<Duration>() {
+            Self::from(d).into_bound_py_any(py)
+        } else {
+            use ryo3_macro_rules::any_repr;
+
+            let valtype = any_repr!(value);
+            py_type_err!("Duration conversion error: {valtype}")
+        }
+    }
+
+    #[cfg(feature = "pydantic")]
+    #[staticmethod]
+    fn _pydantic_validate<'py>(
+        value: &Bound<'py, PyAny>,
+        _handler: &Bound<'py, PyAny>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        Self::from_any(value).map_err(|e| {
+            use ryo3_macro_rules::py_value_error;
+            py_value_error!("Duration validation error: {e}")
+        })
+    }
+
+    #[cfg(feature = "pydantic")]
+    #[classmethod]
+    fn __get_pydantic_core_schema__<'py>(
+        cls: &Bound<'py, ::pyo3::types::PyType>,
+        source: &Bound<'py, PyAny>,
+        handler: &Bound<'py, PyAny>,
+    ) -> PyResult<Bound<'py, PyAny>> {
+        use ryo3_pydantic::GetPydanticCoreSchemaCls;
+        Self::get_pydantic_core_schema(cls, source, handler)
+    }
 }
 
 impl std::fmt::Debug for PyDuration {
@@ -658,10 +879,31 @@ impl std::fmt::Debug for PyDuration {
     }
 }
 
-#[derive(Debug, Clone, FromPyObject)]
-enum PyDurationComparable {
-    PyDuration(PyDuration),
-    Duration(Duration),
+#[cfg(feature = "jiff")]
+impl FromStr for PyDuration {
+    type Err = jiff::Error;
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        TEMPORAL_SPAN_PARSER
+            .parse_unsigned_duration(s)
+            .map(Self::from)
+            .or_else(|_| {
+                FRIENDLY_SPAN_PARSER
+                    .parse_unsigned_duration(s)
+                    .map(Self::from)
+            })
+    }
+}
+
+#[cfg(not(feature = "jiff"))]
+impl FromStr for PyDuration {
+    type Err = ryo3_macro_rules::PyErr;
+
+    fn from_str(_s: &str) -> Result<Self, Self::Err> {
+        use ryo3_macro_rules::py_not_implemented_error;
+        Err(py_not_implemented_error!(
+            "Duration::from_str is not implemented; enable the 'jiff' feature to use this functionality"
+        ))
+    }
 }
 
 // ----------------------------------------------------------------------------
@@ -675,3 +917,36 @@ mod interns {
     py_intern_fn!(secs);
     py_intern_fn!(nanos);
 }
+
+#[cfg(feature = "pydantic")]
+mod pydantic {
+    use super::PyDuration;
+    use pyo3::prelude::*;
+    use pyo3::types::PyType;
+    use ryo3_pydantic::{GetPydanticCoreSchemaCls, interns};
+
+    impl GetPydanticCoreSchemaCls for PyDuration {
+        fn get_pydantic_core_schema<'py>(
+            cls: &Bound<'py, PyType>,
+            source: &Bound<'py, PyAny>,
+            _handler: &Bound<'py, PyAny>,
+        ) -> PyResult<Bound<'py, PyAny>> {
+            let py = source.py();
+            let core_schema = ryo3_pydantic::core_schema(py)?;
+            let timedelta_schema =
+                core_schema.call_method(interns::timedelta_schema(py), (), None)?;
+            let validation_fn = cls.getattr(interns::_pydantic_validate(py))?;
+            let args = pyo3::types::PyTuple::new(py, vec![&validation_fn, &timedelta_schema])?;
+            let string_serialization_schema =
+                core_schema.call_method(interns::to_string_ser_schema(py), (), None)?;
+            let serialization_kwargs = pyo3::types::PyDict::new(py);
+            serialization_kwargs
+                .set_item(interns::serialization(py), &string_serialization_schema)?;
+            core_schema.call_method(
+                interns::no_info_wrap_validator_function(py),
+                args,
+                Some(&serialization_kwargs),
+            )
+        }
+    }
+}
