@@ -18,28 +18,23 @@ fn extract_regex(regex: &Bound<PyAny>) -> PyResult<PyRegex> {
 
 #[pyfunction]
 #[pyo3(signature= (regex, path=None))]
-pub fn which_re(regex: &Bound<'_, PyAny>, path: Option<&str>) -> PyResult<Vec<PathBuf>> {
+pub fn which_re(
+    py: Python<'_>,
+    regex: &Bound<'_, PyAny>,
+    path: Option<&str>,
+) -> PyResult<Vec<PathBuf>> {
     let regex = extract_regex(regex)?;
     if let Some(p) = path {
-        match ::which::which_re_in(regex, Some(p)) {
-            Ok(p) => {
-                let which_vec = p.into_iter().collect::<Vec<PathBuf>>();
-                Ok(which_vec)
-            }
-            Err(e) => Err(PyErr::new::<pyo3::exceptions::PyValueError, _>(format!(
-                "which-re-err: {e}"
-            ))),
-        }
+        // let do with detach
+        py.detach(|| {
+            ::which::which_re_in(regex, Some(p)).map(|p| p.into_iter().collect::<Vec<PathBuf>>())
+        })
+        .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("which-re-err: {e}")))
     } else {
-        match ::which::which_re(regex) {
-            Ok(p) => {
-                let which_vec = p.into_iter().collect::<Vec<PathBuf>>();
-                Ok(which_vec)
-            }
-            Err(_e) => Err(PyErr::new::<pyo3::exceptions::PyOSError, _>(
-                "which: current directory is not a valid path",
-            )),
-        }
+        py.detach(|| ::which::which_re(regex).map(|p| p.into_iter().collect::<Vec<PathBuf>>()))
+            .map_err(|e| {
+                PyErr::new::<pyo3::exceptions::PyValueError, _>(format!("which-re-err: {e}"))
+            })
     }
 }
 
