@@ -32,14 +32,15 @@ impl<'py> FromPyObject<'_, 'py> for PyCompressionLevel {
 
     fn extract(obj: Borrowed<'_, 'py, PyAny>) -> Result<Self, Self::Error> {
         if let Ok(level) = obj.extract::<i32>() {
-            if !(1..=22).contains(&level) {
-                return Err(PyErr::new::<pyo3::exceptions::PyValueError, _>(format!(
+            Self::try_from(level).map_err(|()| {
+                PyErr::new::<pyo3::exceptions::PyValueError, _>(format!(
                     "Invalid compression level: {level}. Must be between 1 and 22."
-                )));
-            }
-            Ok(Self(level))
+                ))
+            })
         } else {
-            Ok(Self(::zstd::DEFAULT_COMPRESSION_LEVEL))
+            Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(
+                "Compression level must be an integer between 1 and 22.",
+            ))
         }
     }
 }
