@@ -2,7 +2,6 @@
 //!
 //! makes a submodule `_dev` and renames it to `ry.ryo3._dev` containing all
 //! dev exports
-// use crate::ry::dev_json;
 use pyo3::prelude::*;
 use pyo3::types::PyDict;
 use pyo3::{intern, wrap_pymodule};
@@ -10,7 +9,6 @@ use pyo3::{intern, wrap_pymodule};
 #[pymodule(gil_used = false, submodule, name = "_dev")]
 pub fn dev(m: &Bound<'_, PyModule>) -> PyResult<()> {
     ryo3_dev::pymod_add(m)?;
-    // dev_json::pymod_add(m)?;
     Ok(())
 }
 
@@ -21,9 +19,7 @@ pub fn pymod_add(m: &Bound<'_, PyModule>) -> PyResult<()> {
 
     // renaming
     let sys = PyModule::import(py, intern!(py, "sys"))?;
-    let sys_modules = sys
-        .getattr(intern!(py, "modules"))?
-        .downcast_into::<PyDict>()?;
+    let sys_modules = sys.getattr(intern!(py, "modules"))?.cast_into::<PyDict>()?;
 
     // dev module
     let pystr_dev = intern!(py, "_dev");
