@@ -4,7 +4,7 @@ import pytest
 
 import ry
 
-STDNET_OBJS = [
+_STDNET_OBJS = [
     (ry.Ipv4Addr("192.168.0.1"), '"192.168.0.1"'),
     (ry.Ipv6Addr("::1"), '"::1"'),
     (ry.IpAddr("192.168.0.1"), '"192.168.0.1"'),
@@ -16,7 +16,7 @@ STDNET_OBJS = [
 ]
 
 
-@pytest.mark.parametrize("obj_expected", STDNET_OBJS)
+@pytest.mark.parametrize("obj_expected", _STDNET_OBJS)
 def test_stringify_std_net_objs(
     obj_expected: tuple[
         ry.Ipv4Addr
@@ -41,7 +41,27 @@ def test_stringify_duration() -> None:
     assert ry.stringify(ry.Duration(secs=1)).decode() == '"PT1S"'
 
 
+def test_stringify_duration_min_max() -> None:
+    data = {
+        "min": ry.Duration.MIN,
+        "max": ry.Duration.MAX,
+    }
+    json_string = ry.stringify(data).decode()
+    assert json_string == '{"min":"PT0S","max":"PT5124095576030431H15.999999999S"}'
+    assert ry.parse_json(json_string) == {
+        "min": ry.Duration.MIN.isoformat(),
+        "max": ry.Duration.MAX.isoformat(),
+    }
+
+
 def test_stringify_duration_max() -> None:
+    assert (
+        ry.stringify(ry.Duration.MAX).decode() == '"PT5124095576030431H15.999999999S"'
+    )
+
+
+@pytest.mark.skip(reason="Duration.MAX serialization changed to ISO8601 format")
+def test_stringify_duration_max_dict() -> None:
     assert (
         ry.stringify(ry.Duration.MAX).decode()
         == '{"secs":18446744073709551615,"nanos":999999999}'
