@@ -1,10 +1,9 @@
 from __future__ import annotations
 
+import tomllib
 from pathlib import Path
 
-import tomli
-
-import ry as ry
+import ry
 
 PWD = Path(__file__).parent
 PYPROJECT_TOML = PWD.parent.parent / "pyproject.toml"
@@ -20,22 +19,13 @@ def _repo_root() -> Path:
     raise RuntimeError(msg)
 
 
-REPO_ROOT = _repo_root()
-
-
-def _version_from_cargo_toml() -> str:
-    Path("Cargo.toml").read_text()
-    cargo_version = tomli.loads(Path("Cargo.toml").read_text())["package"]["version"]
-    if not isinstance(cargo_version, str):
-        msg = f"Cargo version is not a string: {cargo_version}"
-        raise RuntimeError(msg)
-    return cargo_version
+_REPO_ROOT = _repo_root()
 
 
 def _version_from_workspace_package() -> str:
-    root_cargo_toml_filepath = REPO_ROOT / "Cargo.toml"
+    root_cargo_toml_filepath = _REPO_ROOT / "Cargo.toml"
     s = root_cargo_toml_filepath.read_text()
-    version = tomli.loads(s)["workspace"]["package"]["version"]
+    version = tomllib.loads(s)["workspace"]["package"]["version"]
     if not isinstance(version, str):
         msg = f"Cargo version is not a string: {version}"
         raise RuntimeError(msg)
